import { PROJECT_ID } from '@/constants';
import { backgroundsFixture } from '@/models/fixtures/backgroundsFixture';
import { modelsFixture } from '@/models/fixtures/modelsFixture';
import { overlaysFixture } from '@/models/fixtures/overlaysFixture';
import { QueryData } from '@/types/query';
import { mockNetworkResponse } from '@/utils/mockNetworkResponse';
import { getReduxWrapperWithStore } from '@/utils/testing/getReduxWrapperWithStore';
import { HttpStatusCode } from 'axios';
import { apiPath } from '../apiPath';
import { backgroundsDataSelector } from '../backgrounds/background.selectors';
import { modelsDataSelector } from '../models/models.selectors';
import { overlaysDataSelector } from '../overlays/overlays.selectors';
import { AppDispatch, StoreType } from '../store';
import { initMapData } from './map.thunks';
import { InitMapDataActionPayload } from './map.types';

const mockedAxiosClient = mockNetworkResponse();

const EMPTY_QUERY_DATA: QueryData = {
  modelId: undefined,
  backgroundId: undefined,
  initialPosition: undefined,
};

describe('map thunks', () => {
  describe('initMapData - thunk', () => {
    describe('when API is returning valid data', () => {
      let store = {} as StoreType;
      let payload = {} as InitMapDataActionPayload;

      beforeAll(async () => {
        mockedAxiosClient.resetHandlers();
        mockedAxiosClient.onGet(apiPath.getModelsString()).reply(HttpStatusCode.Ok, modelsFixture);
        mockedAxiosClient
          .onGet(apiPath.getAllOverlaysByProjectIdQuery(PROJECT_ID, { publicOverlay: true }))
          .reply(HttpStatusCode.Ok, overlaysFixture);
        mockedAxiosClient
          .onGet(apiPath.getAllBackgroundsByProjectIdQuery(PROJECT_ID))
          .reply(HttpStatusCode.Ok, backgroundsFixture);

        store = getReduxWrapperWithStore().store;
        const dispatch = store.dispatch as AppDispatch;
        payload = (await dispatch(initMapData({ queryData: EMPTY_QUERY_DATA })))
          .payload as InitMapDataActionPayload;
      });

      it('should fetch backgrounds data in store', async () => {
        const data = backgroundsDataSelector(store.getState());
        expect(data).toEqual(backgroundsFixture);
      });

      it('should fetch overlays data in store', async () => {
        const data = overlaysDataSelector(store.getState());
        expect(data).toEqual(overlaysFixture);
      });

      it('should fetch models data in store', async () => {
        const data = modelsDataSelector(store.getState());
        expect(data).toEqual(modelsFixture);
      });

      it('should return valid payload', () => {
        const FIRST = 0;

        expect(payload).toMatchObject({
          modelId: modelsFixture[FIRST].idObject,
          backgroundId: backgroundsFixture[FIRST].id,
        });
      });
    });

    describe('when API is returning empty array', () => {
      let store = {} as StoreType;
      let payload = {} as InitMapDataActionPayload;

      beforeEach(async () => {
        mockedAxiosClient.onGet(apiPath.getModelsString()).reply(HttpStatusCode.Ok, []);
        mockedAxiosClient
          .onGet(apiPath.getAllOverlaysByProjectIdQuery(PROJECT_ID, { publicOverlay: true }))
          .reply(HttpStatusCode.Ok, []);
        mockedAxiosClient
          .onGet(apiPath.getAllBackgroundsByProjectIdQuery(PROJECT_ID))
          .reply(HttpStatusCode.Ok, []);

        store = getReduxWrapperWithStore().store;
        const dispatch = store.dispatch as AppDispatch;
        payload = (await dispatch(initMapData({ queryData: EMPTY_QUERY_DATA })))
          .payload as InitMapDataActionPayload;
      });

      it('should return empty payload', () => {
        expect(payload).toStrictEqual({});
      });
    });
  });
});
